#!/usr/bin/python3

import os
import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt


# ==== Get Log Plots ====
def create_overview_plot(logging, out_dir, file_type="png"):
    """
    Overview plot for logging data of proharmed's functions

    :param logging: overview logging Dataframe that has been returned by one of the four methods of proharmed
    :param out_dir: (optional) output directory to save the plots
    :param file_type: (optional) file type for the plots (png, pdf, jpg, ...)
    :return:
    """
    # ==== Prepare dataframe ====
    logging_names = [x for x in logging.columns if x.startswith(("Nr"))]
    # ==== Create box plot ====
    df = pd.melt(logging[logging_names])
    fig = plt.figure(figsize=(6, 6), dpi=80)
    ax = sns.boxplot(data=df, x="variable", y="value")
    ax.set(title="Distribution of the number of entries per line",
           ylabel="Number of entries per line", xlabel="")
    plt.xticks(rotation=45)
    fig.tight_layout()
    fig.savefig(os.path.join(out_dir, f"overview_log_box.{file_type}"), bbox_inches='tight')
    # ==== Create bar plot ====
    df = logging[logging_names].sum().to_frame().reset_index(level=0)
    # ==== Create plot ====
    fig = plt.figure(figsize=(6, 6), dpi=80)
    ax = sns.barplot(data=df, x="index", y=0)
    ax.set(title="Number of entries added and removed",
           ylabel="Number of entries", xlabel="")
    ax.bar_label(ax.containers[0])
    plt.xticks(rotation=45)
    fig.tight_layout()
    fig.savefig(os.path.join(out_dir, f"overview_log_bar.{file_type}"), bbox_inches='tight')


def create_filter_detailed_plot(logging, organism, out_dir, file_type="png"):
    """
    Detailed plot for logging data of proharmed's filter_protein_ids method

    :param logging: detailed logging Dataframe that has been returned by the filtered_protein_ids method of proharmed
    :param organism: organism that has been specified in the filter_protein_ids method of proharmed
    :param out_dir: (optional) output directory to save the plot
    :param file_type: (optional) file type for the plot
    :return:
    """
    organisms = {"human": "Homo sapiens (Human)", "rat": "Rattus norvegicus (Rat)",
                 "mouse": "Mus musculus (Mouse)", "rabbit": "Oryctolagus cuniculus (Rabbit)"}
    # ==== Prepare dataframe ====
    df_dict = dict()
    if logging.empty:
        df_dict["Decoys"] = 0
        df_dict["Unreviewed"] = 0
        df_dict["Not found IDs"] = 0
        df_dict["Wrong organism"] = 0
    else:
        df_dict["Decoys"] = len(logging[logging["Organism"] == "Decoy"].index)
        df_dict["Unreviewed"] = len(
            logging[(logging["Reviewed"] == "unreviewed") & (logging["Organism"] == organisms[organism])].index)
        df_dict["Not found IDs"] = len(logging[logging["Organism"] == "Not found"].index)
        df_dict["Wrong organism"] = len(
            logging[~logging["Organism"].isin(["Not found", "Decoy", organisms[organism]])].index)
    df = pd.melt(pd.DataFrame(df_dict, index=[0]))
    # ==== Create plot ====
    fig = plt.figure(figsize=(6, 6), dpi=80)
    ax = sns.barplot(data=df, x="variable", y="value")
    ax.set(title="Number of entries removed by cause",
           ylabel="Number of entries", xlabel="")
    ax.bar_label(ax.containers[0])
    fig.tight_layout()
    fig.savefig(os.path.join(out_dir, f"detailed_log.{file_type}"), bbox_inches='tight')


def create_reduced_detailed_plot(logging, out_dir, file_type="png"):
    """
    Detailed plot for logging data of proharmed's reduce_genenames method

    :param logging: detailed logging Dataframe that has been returned by the reduce_genenames method of proharmed
    :param out_dir: (optional) output directory to save the plot
    :param file_type: (optional) file type for the plot
    :return:
    """
    # ==== Prepare dataframe ====
    df_dict = dict()
    if logging.empty:
        df_dict["Not found IDs"] = 0
        df_dict["Different Name"] = 0
    else:
        df_dict["Not found IDs"] = len(logging[logging["Reduced Gene Name"] == "Not found"].index)
        df_dict["Different Name"] = len(
            logging[logging["Reduced Gene Name"] != "Not found"].index)
    df = pd.melt(pd.DataFrame(df_dict, index=[0]))
    # ==== Create plot ====
    fig = plt.figure(figsize=(6, 6), dpi=80)
    ax = sns.barplot(data=df, x="variable", y="value")
    ax.set(title="Number of entries removed by cause",
           ylabel="Number of entries", xlabel="")
    ax.bar_label(ax.containers[0])
    fig.tight_layout()
    fig.savefig(os.path.join(out_dir, f"detailed_log.{file_type}"), bbox_inches='tight')


def create_ortholog_detailed_plot(logging, organism, out_dir, file_type="png"):
    """
    Detailed plot for logging data of proharmed's map_orthologs method

    :param logging: detailed logging Dataframe that has been returned by the map_orthologs method of proharmed
    :param organism: organism that has been specified in the map_orthologs method of proharmed
    :param out_dir: (optional) output directory to save the plot
    :param file_type: (optional) file type for the plot    :return:
    """
    # ==== Prepare dataframe ====
    df_dict = dict()
    if logging.empty:
        df_dict["Not found target names"] = 0
        df_dict["Not found target IDs"] = 0
        df_dict["Not found source IDs"] = 0
        df_dict["Wrong organism"] = 0
    else:
        df_dict["Not found target names"] = len( logging[(logging["ortholog_ensg"] != "") & (logging["target_symbol"] == "")].index)
        df_dict["Not found target IDs"] = len( logging[(logging["ortholog_ensg"] == "") & (logging["target_symbol"] == "")].index)
        df_dict["Not found source IDs"] = len(logging[logging["ortholog_ensg"] == "Not found"].index)
        df_dict["Wrong organism"] = len(
            logging[~logging["source_organism"].isin(["Not found", organism])].index)
    df = pd.melt(pd.DataFrame(df_dict, index=[0]))
    # ==== Create plot ====
    fig = plt.figure(figsize=(6, 6), dpi=80)
    ax = sns.barplot(data=df, x="variable", y="value")
    ax.set(title="Number of entries removed by cause",
           ylabel="Number of entries", xlabel="")
    ax.bar_label(ax.containers[0])
    plt.xticks(rotation=45)
    fig.tight_layout()
    fig.savefig(os.path.join(out_dir, f"detailed_log.{file_type}"), bbox_inches='tight')



#!/usr/bin/python3


import httplib2 as http
import json
import pandas as pd
from ratelimit import limits, sleep_and_retry

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

# 10 calls per second
CALLS = 10
RATE_LIMIT = 1


@sleep_and_retry
@limits(calls=CALLS, period=RATE_LIMIT)
def get_HGNC_mapping(id, request="symbol"):
    headers = {
        'Accept': 'application/json',
    }

    output = pd.DataFrame()

    uri = 'http://rest.genenames.org'
    path = '/fetch/' + request + '/' + id

    target = urlparse(uri + path)
    method = 'GET'
    body = ''

    h = http.Http()

    response, content = h.request(
        target.geturl(),
        method,
        body,
        headers)

    if response['status'] == '200':
        status = "success"
    else:
        print('Error detected: ' + response['status'])
        status = "error"
        return None

    if status == "success":
        data = json.loads(content)

        if (len(data["response"]["docs"]) == 0):
            return None
        else:
            for entry in data["response"]["docs"]:
                hgnc_id = entry.get("hgnc_id", None)
                symbol = entry.get("symbol", None)
                prev = entry.get("prev_symbol", None)
                alias = entry.get("alias_symbol", None)

                if prev is not None:
                    prev = ";".join(prev)

                if alias is not None:
                    alias = ";".join(alias)

                output_chunk = pd.Series([request, id, hgnc_id, symbol, prev, alias])
                output = pd.concat([output, output_chunk], axis=1)

            output = output.T
            output.columns = ["Request Type", "Input", "HGNC ID", "Symbol", "Previous Symbol", "Alias Symbol"]
            return (output)



#!/bin/python3

import os
import sys
import argparse
import time
import psutil
import pandas as pd
import csv
from pathlib import Path

start_time = time.time()


def save_parameters(script_desc: str, arguments):
    """
    Save command line options into local variables.

    :return: values assigned to input arguments
    """
    descr = "\n############################################################################\n"
    descr += "################# MaxQuantHandler - %(prog)s ##################\n"
    descr += script_desc
    descr += "\n############################################################################\n"
    descr += "\nusage: python3 %(prog)s [required arguments] [optional arguments]\n"
    epilo = _get_epilog(script_name=os.path.basename(sys.argv[0]))
    parser = argparse.ArgumentParser(description=descr, formatter_class=argparse.RawTextHelpFormatter, epilog=epilo,
                                     usage=argparse.SUPPRESS, add_help=False)
    required_args = parser.add_argument_group("required arguments")
    if 'd' in arguments:
        required_args.add_argument('-d', '--data', type=str, help='Data file', default=None, required=True)
    if 'f_req' in arguments:
        required_args.add_argument('-f', '--fasta_file', type=str, help='Fasta file', required=True)
    if 'or_req' in arguments:
        required_args.add_argument('-or', '--organism', choices=["human", "mouse", "rat", "rabbit"], type=str,
                                   required=True, help='Specify organism the ids should match to.')
    if 'tor_req' in arguments:
        required_args.add_argument('-tor', '--tar_organism', choices=["human", "mouse", "rat", "rabbit"], type=str,
                                   required=True, help='Specify organism from which orthologs should be mapped.')
    if 'pc_req' in arguments:
        required_args.add_argument('-pc', '--protein_column', type=str,
                                   help='Name of column with protein IDs.', required=True)
    if 'gc_req' in arguments:
        required_args.add_argument('-gc', '--gene_column', type=str, default=None,
                                   help='Name of column with gene names.', required = True)
    if 'm' in arguments:
        required_args.add_argument('-m', '--mode',
                                   choices=['all', 'fasta', 'uniprot', 'uniprot_one', 'uniprot_primary'],
                                   type=str, required=True, help='Mode of refilling. See below for more infos.')
    if 'rm' in arguments:
        required_args.add_argument( '-m', '--mode',
                                    choices=[ 'ensembl', 'mygeneinfo', 'HGNC', 'enrichment'],
                                    type=str, required=True, help='Mode of reducing. See below for more infos.' )
    if 'i' in arguments:
        required_args.add_argument('-i', '--in_type', choices=['protein', 'gene'], required=True,
                                   help='Define what type should be the source.')

    optional_args = parser.add_argument_group("optional arguments")
    if 'pc' in arguments:
        optional_args.add_argument('-pc', '--protein_column', type=str, default=None,
                                   help='Name of column with protein IDs [Default=None]')
    if 'gc' in arguments:
        optional_args.add_argument('-gc', '--gene_column', type=str, default=None,
                                   help='Name of column with gene names [Default=None]')

    if 'ke' in arguments:
        optional_args.add_argument('-ke', '--keep_empty', action='store_true', default=True,
                                   help = "Bool to indicate whether empty rows should be kept. [Default=True]")
    if 'f' in arguments:
        optional_args.add_argument('-f', '--fasta_file', type=str, help='Fasta file', default=None)
    if 'l' in arguments:
        optional_args.add_argument('-l', '--fill', action='store_false', default=True,
                                   help='Use this flag, if filled values should be skipped. [Default=True]')
    if 'a' in arguments:
        optional_args.add_argument('-a', '--action', type=str, default="delete", choices=['keep', 'delete'],
                                   help='What to do, if IDs cell is empty after filtering. '
                                        'Keep empty cell or delete it.')
    if 'hm' in arguments:
        optional_args.add_argument('-hm', '--hgnc_mode', type=str, choices=["mostfrequent", "all"],
                                   default="mostfrequent",
                                   help="What to do if reduce_mode is HGNC. Take most frequent gene name or all. "
                                        "[Default=mostfrequent]")
    if 'rc' in arguments:
        optional_args.add_argument('-rc', '--res_column', type=str, default=None,
                                   help='Name of output column. If None, input column will be edited. [Default = None]')
    if 'or' in arguments:
        optional_args.add_argument('-or', '--organism', choices=["human", "mouse", "rat", "rabbit"], type=str,
                                   default=None, help='Specify organism the ids should match to.')
    if 'r' in arguments:
        optional_args.add_argument('-r', '--reviewed', action='store_true', default=False,
                                   help='Bool to indicate if only reviewed protein IDs should be kept.')
    if 'rv' in arguments:
        optional_args.add_argument('-rv', '--rev_con', action='store_true', default=False,
                                   help='Set flag if decoy and contaminants IDs (REV__, CON__) should be kept.')
    if 'o' in arguments:
        optional_args.add_argument('-o', '--out_dir', type=str, default='./', help='Output directory. [Default=./]')
    optional_args.add_argument("-h", "--help", action="help", help="show this help message and exit")
    args = parser.parse_args()
    if 'd' in arguments:
        args.data = pd.read_table(args.data).fillna("")
        args.file_name = Path(args.single_file).stem
    return args


def _get_epilog(script_name):
    epilog = ""
    if script_name == 'remap_genenames.py':
        epilog += "\n----------------------------------------------------------------------------\n"
        epilog += "\nsupported modes\n"
        epilog += "  all\t\t\tUse primarly fasta infos and additionally uniprot infos.\n"
        epilog += "  fasta\t\t\tUse information extracted from fasta headers.\n"
        epilog += "  uniprot\t\tUse mapping information from uniprot and use all gene names.\n"
        epilog += "  uniprot_primary\tUse mapping information from uniprot and only all primary gene names.\n"
        epilog += "  uniprot_one\t\tUse mapping information from uniprot and only use most frequent single gene name.\n"
    if script_name == "reduce_genenames.py":
        epilog += "\n----------------------------------------------------------------------------\n"
        epilog += "\nsupported modes\n"
        epilog += "  ensembl\t\tUse gProfiler to reduce gene names to those have an Ensembl ID.\n"
        epilog += "  mygeneinfo\t\tUse mygeneinfo database to reduce gene names to those having an entry in mygeneinfo.\n"
        epilog += "  HGNC\t\tUse HGNC database to reduce gene names to those having an entry in HGNC (only for human).\n"
        epilog += "  enrichment\tUse gProfiler to reduce gene names to those having a functional annotation.\n"
    epilog += "\n############################################################################\n"
    return epilog


def print_current_usage(text):
    memory_usage = '{0:.2f}'.format(psutil.Process(os.getpid()).memory_info().rss / 1024 ** 2)
    time_usage = time.strftime("%H:%M:%S", time.gmtime(time.time() - start_time))
    print('[{}|{}MB] '.format(time_usage, memory_usage) + text)


def find_delimiter(filename):
    sniffer = csv.Sniffer()
    with open(filename) as fp:
        delimiter = sniffer.sniff(fp.readline()).delimiter
    return delimiter



#!/usr/bin/python3

import pandas as pd
import itertools
from .mq_utils.logger import get_reduced_genenames_logging
from .mq_utils import mapping_handler as mh, runner_utils as ru
from pathlib import Path
import csv


def reduce_genenames(data: pd.DataFrame, gene_column: str, mode:str, organism: str,
                     res_column: str = None, keep_empty: bool = True, HGNC_mode: str = "mostfrequent"):
    """
    Reduce gene names in data file based on chosen mode.

    :param data: Dataframe containing a column with gene names
    :param mode: Mode on how to reduce gene names
    :param gene_column: Column name with gene names
    :param res_column: Set column name for reduced results. If None, the gene_column will be overridden.
    :param keep_empty: Set True if rows with no gene names should be kept
    :param organism: Organism to map to
    :param HGNC_mode: Mode on how to select the gene names in HGNC (mostfrequent, all)
    :param return_log: Set True if log dataframes should be returned

    :return: Reduced data as dataframe
    """
    data_copy = data.copy(deep=True)
    data_copy[gene_column] = data_copy[gene_column].astype("string")

    handler = mh.MappingHandler()
    # ==== Preload info for all IDs ====
    handler.get_mapping(ids=";".join(data_copy[gene_column]).split(";"),
                        in_type="reduced_genes", organism=organism, reduction_mode=mode)

    # ==== If gene_column not in data ====
    if gene_column not in data_copy.columns:
        raise Exception("Gene Column Not in Data Column!")

    # ==== If Mode is HGNC check if organism is Human ====
    if (mode == "HGNC") and (organism != "human"):
        raise Exception("HGNC Database only for Human Genes!")

    # ==== Reduce Gene Names ====
    reduced_gene_names = data_copy[gene_column].apply(
        lambda row: get_reduced_genenames(ids=row.split(";"), handler=handler,
                                          reduction_mode=mode, HGNC_mode=HGNC_mode, organism=organism))

    # ==== Logging ====
    log_dict = get_reduced_genenames_logging(data_copy[gene_column], reduced_gene_names, handler, organism, mode)

    # ==== If target column depending if res_column is set ====
    column = res_column if res_column is not None else gene_column

    # ==== Set Reduced Gene Names To DataFrame ====
    data_copy[column] = reduced_gene_names

    # ==== Remove Rows with Empty Gene Names ====
    if keep_empty is False:
        data_copy = data_copy[data_copy[column] != ""]  # remove

    return data_copy, log_dict


def get_reduced_genenames(ids, handler, organism=None, reduction_mode="ensembl", HGNC_mode="mostfrequent"):
    mapping = handler.get_mapping(ids=ids, in_type="reduced_genes", organism=organism, reduction_mode=reduction_mode)
    if mapping.empty:
        return ""
    else:
        # check orga
        if reduction_mode == "HGNC":
            # separate case because we have two modes (mostfrequent and all)
            HGNC_output = mapping[mapping["Mode"] == reduction_mode].dropna()
            HGNC_list = list(itertools.chain.from_iterable(list(HGNC_output["Reduced Gene Name"])))
            if HGNC_mode == "mostfrequent":
                reduced_genenames = list(pd.Series(HGNC_list).mode())
            elif HGNC_mode == "all":
                reduced_genenames = HGNC_list
        else:
            # remove None
            reduced_genenames = list(mapping[-mapping["Reduced Gene Name"].isin(["None", None])][
                                         "Reduced Gene Name"])  # "None" and None because Ensembl returns for example "None"
        return ";".join(list(set(reduced_genenames)))


if __name__ == "__main__":
    description = "                  Reduce gene names in data file."
    parameters = ru.save_parameters(script_desc=description,
                                    arguments=('d', 'gc_req', 'rm', 'or_req', 'rc', 'ke', 'hm', 'o'))
    res, log = reduce_genenames(data=parameters.data, gene_column=parameters.gene_column, mode=parameters.mode,
                                organism=parameters.organism, res_column = parameters.res_column,
                                keep_empty=parameters.keep_empty, HGNC_mode= parameters.hgnc_mode)
    res.to_csv(parameters.out_dir + Path(parameters.file_name).stem + "_reduced.txt", header=True,
               index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")

    log["Overview_Log"].to_csv(parameters.out_dir + Path(parameters.file_name).stem + "_reduced_overview_log.txt",
                               header = True, index = False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")
    log["Detailed_Log" ].to_csv(
        parameters.out_dir + Path( parameters.file_name ).stem + "_reduced_detailed_log.txt",
        header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" " )



#!/usr/bin/python3

import re
import pandas as pd
from .mq_utils import runner_utils as ru


def grep_header_info(fasta: str) -> pd.DataFrame:
    """
    Grep information from headers in given fasta file.

    :param fasta: Fasta file
    :return: Grepped information as dataframe with columns := 'uniprot','description','symbol','organism','ox','pe','sv'
    """
    infos = list()
    with open(fasta) as file:
        for line in file:
            if line.startswith('>'):
                # get gene name
                matches = re.search("GN=(.*)\sPE", line)
                gene_name = matches.group(1) if matches else ""
                gene_string = "GN=.*\s" if matches else ""
                # get full info
                matches = re.search("\|(.*)\|\S*\s(.*)\sOS=(.*)\sOX=(.*)\s" + gene_string + "PE=(.*)\sSV=(.*)", line)
                infos.append([matches.group(1), matches.group(2), gene_name, matches.group(3),
                              matches.group(4), matches.group(5), matches.group(6)])
    return pd.DataFrame(infos, columns=['uniprot', 'description', 'symbol', 'organism', 'ox', 'pe', 'sv'])


if __name__ == "__main__":
    description = "                  Grep protein info from fasta file."
    parameters = ru.save_parameters(script_desc=description, arguments=('f_req', 'o'))
    df = grep_header_info(fasta=parameters.fasta_file)
    df.to_csv(parameters.out_dir + "grepped_info.csv", header=True, index=False)



#!/usr/bin/python3

import os
import requests
import pandas as pd
from collections import Counter
from upsetplot import plot
from matplotlib import pyplot as plt


def count_intersection(data: dict, threshold=1):
    full_list = []
    [full_list.extend(data[key]) for key in data]
    intersections = pd.DataFrame.from_dict(Counter(full_list), orient='index').reset_index()
    intersections = intersections.sort_values(by=[0], ascending=False).rename(columns={"index": "ID", 0: "Count"})
    return intersections[intersections["Count"] >= threshold]


def plot_intersections(data: dict, out_dir, file_type="png"):
    names = list()
    all_elements = set()
    data_sets = dict()
    for key in data:
        names.append(key)
        data_sets[key] = set(data[key])
        all_elements = all_elements.union(data_sets[key])
    df = pd.DataFrame([[element in data_sets[key] for key in data_sets] for element in all_elements], columns=names)
    df_up = df.groupby(names).size()

    fig = plt.figure(figsize=(12, 6), dpi=80)
    plot(df_up, fig=fig, element_size=None, show_counts=True, orientation='horizontal')
    plt.suptitle('Overview of Intersections')
    plt.show()
    fig.savefig(os.path.join(out_dir, f"overview_intersections.{file_type}"), bbox_inches='tight')


def inspect_for_drugs(genes: list):
    url = 'https://api.drugst.one/create_network'
    myobj = {"network": {'nodes': [{"id": gene, "group": "gene"} for gene in genes]}}
    result = requests.post(url, json=myobj)
    return "https://drugst.one?id="+result.json()


def load_multi_files(files: [list, str], columns: [list, str]):
    # Read column names
    if type(columns) is str:
        column_names = pd.read_csv(columns, names=['columns', 'file'])
    elif type(columns) is list:
        column_names = pd.DataFrame(columns, columns=['columns']).reset_index()
    else:
        column_names = pd.DataFrame()

    # Read files
    data = dict()
    if type(files) is list:
        for index, file in enumerate(files):

            if "file" in column_names.columns:
                col = column_names[column_names['file'] == os.path.basename(file)]["columns"].iloc[0]
            elif "index" in column_names.columns:
                col = column_names[column_names['index'] == index]["columns"].iloc[0]
            else:
                col = None

            if type(file) is pd.DataFrame:
                if col is not None:
                    data[str(index)] = file[col].to_list()
                else:
                    data[str(index)] = file.iloc[:, 0].to_list()
            elif type(file) is list:
                data[str(index)] = file
            elif os.path.isfile(file):
                if col is not None:
                    data[os.path.basename(file)] = pd.read_csv(file)[col].to_list()
                else:
                    data[os.path.basename(file)] = pd.read_csv(file).iloc[:, 0].to_list()
    elif type(files) is str:
        if os.path.isfile(files):
            if "file" in column_names.columns:
                col = column_names[column_names['file'] == os.path.basename(files)]["columns"].iloc[0]
                data[os.path.basename(files)] = pd.read_csv(files)[col].to_list()
            elif "index" in column_names.columns:
                col = column_names[column_names['index'] == 0]["columns"].iloc[0]
                data[os.path.basename(files)] = pd.read_csv(files)[col].to_list()
            else:
                data[os.path.basename(files)] = pd.read_csv(files).iloc[:, 0].to_list()
        elif os.path.isdir(files):
            if "file" in column_names.columns:
                for file in column_names["file"]:
                    col = column_names[column_names['file'] == os.path.basename(file)]["columns"].iloc[0]
                    data[os.path.basename(file)] = pd.read_csv(os.path.join(files, file))[col].to_list()
            else:
                for index, file in enumerate(os.listdir(files)):
                    if "index" in column_names.columns:
                        col = column_names[column_names['index'] == index]["columns"].iloc[0]
                        data[os.path.basename(file)] = pd.read_csv(os.path.join(files, file))[col].to_list()
                    else:
                        data[os.path.basename(file)] = pd.read_csv(os.path.join(files, file)).iloc[:, 0].to_list()

    # split and explode
    for data_key in data:
        seen = set()
        data[data_key] = [x for x in ";".join(data[data_key]).split(";") if x not in seen and not seen.add(x)]

    return data



#!/usr/bin/python3

import pandas as pd


# ==== Logging DataFrame For Filtering Ids ====
def get_filter_ids_logging(original, filtered, handler, organism):
    # ==== create dataframe with for each row original ids, filtered ids, nr ids, etc. ====
    log_df = pd.DataFrame({"IDs": original.str.split(";"), "Filtered IDs": filtered.str.split(";")})
    log_df["Removed IDs"] = log_df.apply(
        lambda row: list(set(row["IDs"]).difference(set(row["Filtered IDs"]))), axis=1)
    log_df["Nr IDs"] = log_df["IDs"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Filtered IDs"] = log_df["Filtered IDs"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Removed IDs"] = log_df["Nr IDs"] - log_df["Nr Filtered IDs"]

    # ==== for removed ids --> find out the cause and create df ====
    if sum(log_df["Nr Removed IDs"]) == 0:
        return {"Overview_Log": log_df, "Detailed_Log": pd.DataFrame()}
    else:
        removed_ids = [x for x in log_df["Removed IDs"].explode() if str(x) != 'nan']
        decoy = [x for x in removed_ids if x.startswith(("REV", "CON"))]  # save decoys
        removed_ids_nondecoy = [x for x in removed_ids if (x not in decoy)]
        # ==== Get Information for removed IDs ====
        df, missing = handler.get_preloaded(in_list=removed_ids_nondecoy, in_type="protein")
        df = pd.concat([df, pd.DataFrame({"Protein ID": missing})], ignore_index=True).fillna("Not found")
        df = pd.concat([df, pd.DataFrame({"Protein ID": decoy})], ignore_index=True).fillna("Decoy")
        return {"Overview_Log": log_df, "Detailed_Log": df}


# ==== Logging DataFrame For Remapped Gene Names ====
def get_remapped_genenames_logging(original, remapped, protein_ids, handler, organism):
    # ==== create dataframe with for each row original names, remapped names, nr names, etc. ====
    log_df = pd.DataFrame({"Gene Names": original.str.split(";"), "Remapped Gene Names": remapped.str.split(";")})
    log_df["Added Gene Names"] = log_df.apply(
        lambda row: list(set(row["Remapped Gene Names"]).difference(set(row["Gene Names"]))), axis=1)
    log_df["Removed Gene Names"] = log_df.apply(
        lambda row: list(set(row["Gene Names"]).difference(set(row["Remapped Gene Names"]))), axis=1)
    log_df["Nr Gene Names"] = log_df["Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Remapped Gene Names"] = log_df["Remapped Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Added Gene Names"] = log_df["Added Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Removed Gene Names"] = log_df["Removed Gene Names"].apply(lambda x: len(list(filter(None, x))))

    # ==== for added names --> find out the cause and create df ====
    df, missing = handler.get_preloaded(in_list=protein_ids, in_type="protein", organism=organism)
    df = df[df["Gene Names (primary)"].isin(log_df["Added Gene Names"].explode().to_list())]
    return {"Overview_Log": log_df, "Detailed_Log": df}


# ==== Logging DataFrame For Reducing Gene Names ====
def get_reduced_genenames_logging(original, reduced, handler, organism, mode):
    # ==== Get Information for all original names ====
    df, missing = handler.get_preloaded(in_list=original, in_type="reduced_genes",
                                        organism=organism, reduction_mode=mode)
    # ==== create dataframe with for each row original names, reduced names, nr names, etc. ====
    log_df = pd.DataFrame({"Gene Names": original.str.split(";"), "Reduced Gene Names": reduced.str.split(";")})
    log_df["Added Gene Names"] = log_df.apply(
        lambda row: list(set(row["Reduced Gene Names"]).difference(set(row["Gene Names"]))), axis=1)
    log_df["Removed Gene Names"] = log_df.apply(
        lambda row: list(set(row["Gene Names"]).difference(set(row["Reduced Gene Names"]))), axis=1)
    log_df["Nr Gene Names"] = log_df["Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Reduced Gene Names"] = log_df["Reduced Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Added Gene Names"] = log_df["Added Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Removed Gene Names"] = log_df["Removed Gene Names"].apply(lambda x: len(list(filter(None, x))))

    # ==== for reduced ids --> find out the cause and create df ====
    df = pd.concat([df, pd.DataFrame({"Gene Name": missing})], ignore_index=True).fillna("Not found")
    df = df[df["Gene Name"].isin(log_df["Removed Gene Names"].explode().to_list())]
    return {"Overview_Log": log_df, "Detailed_Log": df}


# ==== Logging DataFrame For Remapped Gene Names ====
def get_ortholog_genenames_logging(original, orthologs, handler, organism, tar_organism):
    # ==== Get Information for all original names ====
    original_names = [x for x in ";".join(original).split(";") if str(x) != 'nan']
    df, missing = handler.get_preloaded(in_list=original_names, in_type="orthologs",
                                        organism=organism, tar_organism=tar_organism)
    df = df.fillna("")
    df = df[df["target_symbol"].isin(["", "N/A"])]
    removed_gene_names = set(df["source_symbol"])

    # ==== create dataframe with for each row original names, ortholog names, nr names, etc. ====
    log_df = pd.DataFrame({"Gene Names": original.str.split(";"), "Ortholog Gene Names": orthologs.str.split(";")})
    log_df["Removed Gene Names"] = log_df.apply(
        lambda row: list(set(row["Gene Names"]).intersection(removed_gene_names)), axis=1)
    log_df["Nr Gene Names"] = log_df["Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Ortholog Gene Names"] = log_df["Ortholog Gene Names"].apply(lambda x: len(list(filter(None, x))))
    log_df["Nr Removed Gene Names"] = log_df["Removed Gene Names"].apply(lambda x: len(list(filter(None, x))))

    # ==== for removed ids --> find out the cause and create df ====
    if len(removed_gene_names) == 0:
        return {"Overview_Log": log_df, "Detailed_Log": pd.DataFrame()}
    else:
        # ==== Get Information for removed names ====
        df = pd.concat([df, pd.DataFrame({"source_symbol": missing})], ignore_index=True).fillna("Not found")
        return {"Overview_Log": log_df, "Detailed_Log": df}


#!/usr/bin/python3

from os.path import dirname, abspath, join
import pandas as pd
from pathlib import Path
from gprofiler import GProfiler
import requests
from .HGNC_mapping import get_HGNC_mapping
import mygene
import numpy as np

here = dirname(dirname(abspath(__file__)))  # needed to make files reachable in python package


class MappingHandler:
    full_protein_mapping = pd.DataFrame(columns=['Gene Names', 'Gene Names (primary)', 'Reviewed', 'Organism',
                                                 'Protein ID'])
    full_ortholog_mapping = pd.DataFrame(columns=['source_symbol', 'source_organism', 'ensg', 'ortholog_ensg',
                                                  'target_symbol', 'target_organism', 'description'])
    full_reduced_gene_mapping = pd.DataFrame(columns=["Gene Name", "Reduced Gene Name", "Organism", "Mode"])

    # def __init__(self, mapping_dir):
    #     mapping_dir = join(here, mapping_dir)
    #     if Path(mapping_dir + "protein_to_genenames.csv").exists():
    #         self.full_protein_mapping = pd.read_csv(mapping_dir + "protein_to_genenames.csv")
    #     if Path(mapping_dir + "genenames_to_orthologs.csv").exists():
    #         self.full_ortholog_mapping = pd.read_csv(mapping_dir + "genenames_to_orthologs.csv")
    #     if Path(mapping_dir + "genenames_to_reduced_genenames.csv").exists():
    #         self.full_reduced_gene_mapping = pd.read_csv(mapping_dir + "genenames_to_reduced_genenames.csv",
    #                                                      na_values=None)

    # === UniProt Mapping ====
    def get_uniprot_mapping(self, ids, organism: str = None):
        """
        Get UniProt mapping for UniProt protein IDs. Optionally filter for organism.

        :param ids: Set of protein IDs
        :param organism: Organism to map to
        :return: dataframe with mapping to each mappable ID
        """
        organisms = {"human": "Homo sapiens (Human)", "rat": "Rattus norvegicus (Rat)", "mouse": "Mus musculus (Mouse)",
                     "rabbit": "Oryctolagus cuniculus (Rabbit)"}
        url = 'https://rest.uniprot.org/uniprotkb/accessions'
        mapping = pd.DataFrame()
        # ==== Get mappings in <= 500 IDs chunks ====
        for i in range(0, len(ids), 500):
            # ==== Remove contaminated and reverse mapped IDs ====
            ids_chunk = [x for x in ids[i:i + 500] if not x.startswith(("REV", "CON"))]
            params = {'format': 'tsv',
                      'accessions': ",".join(ids_chunk),
                      'fields': 'gene_names,gene_primary,reviewed,organism_name,accession'}
            f = requests.get(url=url, params=params)
            # ==== If at least one ID doesn't exist ====
            if f.status_code == 400:
                mapping_chunk = pd.DataFrame()
                # ==== Check each id separately ====
                for id in ids_chunk:
                    params["accessions"] = id
                    f = requests.get(url=url, params=params)
                    if f.status_code != 400:
                        mapping_chunk2 = pd.read_csv(f.url, sep="\t")
                        mapping_chunk = mapping_chunk2 if mapping_chunk.empty \
                            else pd.concat([mapping_chunk, mapping_chunk2])
            # ==== All IDs were mapped ====
            else:
                mapping_chunk = pd.read_csv(f.url, sep="\t")
            # ==== Combine to one mapping dataframe ====
            mapping = mapping_chunk if mapping.empty else pd.concat([mapping, mapping_chunk])
        # ==== Changes inside final mapping dataframe ====
        if not mapping.empty:
            mapping.columns = [*mapping.columns[:-1], 'Protein ID']  # change name of last column
            # ==== Clean accidental white spaces from UniProt ====
            mapping['Gene Names'] = mapping['Gene Names'].str.replace(r';?\s+;?', ';',
                                                                      regex=True)  # change separation to ;
            mapping['Gene Names (primary)'] = mapping['Gene Names (primary)'].str.replace(r'\s+', '', regex=True)
            mapping['Protein ID'] = mapping['Protein ID'].str.replace(r'\s+', '', regex=True)
            # ==== Split and explode to create one row for each ID ====
            mapping['Protein ID'] = mapping['Protein ID'].apply(lambda x: x.split(","))
            mapping = mapping.explode('Protein ID')
            # ==== Save to global mapping ====
            self.full_protein_mapping = pd.concat([self.full_protein_mapping, mapping])
            # ==== Filter for organism if given ====
            if organism is not None:
                mapping = mapping[mapping['Organism'] == organisms[organism]]
        return mapping

    # === Ortholog Mapping ====
    def get_ortholog_mapping(self, ids, organism, tar_organism):
        """
        Get ortholog mapping from source to target organism using gProfiler.

        :param ids: Set of gene names
        :param organism: Organism of the input ids
        :param tar_organism: Organism to map to
        :return: dataframe with mapped ortholog pairs
        """
        organisms = {"human": "hsapiens", "mouse": "mmusculus", "rat": "rnorvegicus", "rabbit": "ocuniculus"}
        gp = GProfiler(return_dataframe=True)
        mapping = gp.orth(organism=organisms[organism], query=ids, target=organisms[tar_organism])
        mapping = mapping.fillna('')
        # ==== Subset and rename columns ====
        mapping = mapping[['incoming', 'converted', 'ortholog_ensg', 'name', 'description']]
        mapping.columns = ['source_symbol', 'ensg', 'ortholog_ensg', 'target_symbol', 'description']
        # ==== Save organism info ====
        mapping.insert(loc=1, column='source_organism', value=organism)
        mapping.insert(loc=5, column='target_organism', value=tar_organism)
        # ==== Save to global mapping ====
        self.full_ortholog_mapping = pd.concat([self.full_ortholog_mapping, mapping])
        return mapping

    # === Reduced Mapping ====
    def get_reduced_mapping(self, ids, organism, reduction_mode="ensembl"):
        if reduction_mode == "ensembl":
            mapping = self.get_ensembl_reduction(ids, organism)  # organism must be set
        elif reduction_mode == "HGNC":
            mapping = self.get_HGNC_reduction(ids)  # organism must be human
        elif reduction_mode == "enrichment":
            mapping = self.get_enrichment_reduction(ids, organism)  # organism must be set
        elif reduction_mode == "mygeneinfo":
            mapping = self.get_mygeneinfo_reduction(ids)  # add for all organisms directly
        else:
            mapping = pd.DataFrame()
        if not mapping.empty:
            mapping["Mode"] = reduction_mode
            self.full_reduced_gene_mapping = pd.concat([self.full_reduced_gene_mapping, mapping])
            return mapping
        else:
            return pd.DataFrame()

    def get_ensembl_reduction(self, ids, organism):  # organism required
        organisms = {"human": "hsapiens", "mouse": "mmusculus", "rat": "rnorvegicus", "rabbit": "ocuniculus"}
        gp = GProfiler(return_dataframe=True)
        gp_df = gp.convert(organism=organisms[organism], query=ids, target_namespace="ENSG")
        if len(gp_df) == 0:
            return pd.DataFrame()
        else:

            # Case 1: in ensembl id and name is None --> remove them
            gp_df = gp_df.drop(gp_df[(gp_df["name"] == "None") & (gp_df["converted"] == "None")].index)

            # Case 2: remove entries that have already an entry in gp_df with a corresponding name
            gp_df = gp_df.drop(gp_df[(gp_df["n_converted"] > 1) & (gp_df["name"] == "None")].index)

            # Case 3: in name is also the ensembl id saved --> save in new name the incoming name else take name
            gp_df["new_name"] = np.where(gp_df["converted"] == gp_df["name"], gp_df["incoming"],
                                         gp_df["name"])

            # Case 4: for entries with ensembl id but without name --> take input name
            gp_df["new_name"] = np.where((gp_df["converted"] != "None") & (gp_df["name"] == "None"),
                                         gp_df["incoming"], gp_df["new_name"])

            mapping = gp_df[["incoming", "new_name"]]
            # Case 5: get entries that have more than 1 entry in gProfiler
            duplicates = mapping[mapping.duplicated("incoming", keep=False)]

            if len(duplicates) > 0:
                # remove them from initial df
                mapping = mapping.drop(duplicates.index)
                # take input names of duplicates as new names
                mapping_chunk = pd.DataFrame(
                    {"incoming": duplicates["incoming"].unique(), "new_name": duplicates["incoming"].unique()})
                mapping = pd.concat([mapping, mapping_chunk])

            # if after removing nothing remains --> return empty dataframe
            if len(mapping) > 0:
                mapping.columns = ["Gene Name", "Reduced Gene Name"]
                mapping.loc[:, "Organism"] = organism
                return mapping
            else:
                return pd.DataFrame()

    def get_HGNC_reduction(self, ids):  # human organism required
        mapping_dict = {}
        for id in ids:
            alias_df = get_HGNC_mapping(id, "alias_symbol")
            symbol_df = get_HGNC_mapping(id, "symbol")
            if alias_df is None and symbol_df is None:
                # no information in HGNC for this id
                mapping_dict[id] = None
            else:
                if alias_df is None:
                    result_df = symbol_df
                elif symbol_df is None:
                    result_df = alias_df
                else:
                    result_df = pd.concat([alias_df, symbol_df])
                mapping_dict[id] = list(result_df["Symbol"])

        mapping = pd.DataFrame({"Gene Name": k, "Reduced Gene Name": v} for k, v in mapping_dict.items())
        mapping["Organism"] = "human"
        return mapping

    def get_enrichment_reduction(self, ids, organism):
        organisms = {"human": "hsapiens", "mouse": "mmusculus", "rat": "rnorvegicus", "rabbit": "ocuniculus"}
        gp = GProfiler(return_dataframe=True)
        # no_evidences = False returns intersections column with ids that match the specific annotation
        gp_df = gp.profile(organism=organisms[organism], query=ids, no_evidences=False)

        if len(gp_df) == 0:
            reduced_ids = [None for id in ids]
        else:
            enrichment_ids = gp_df["intersections"].explode().to_list()
            reduced_ids = [id if id in enrichment_ids else None for id in ids]
        mapping = pd.DataFrame({"Gene Name": ids, "Reduced Gene Name": reduced_ids})
        mapping["Organism"] = organism
        return mapping

    def get_mygeneinfo_reduction(self, ids):
        tax_ids = {"human": 9606, "mouse": 10090, "rat": 10116, "rabbit": 9986}
        inv_tax_ids = {tax_id: organism for organism, tax_id in tax_ids.items()}

        mg = mygene.MyGeneInfo()
        mg_output = mg.querymany(ids, scopes="symbol", fields="symbol,taxid", as_dataframe=True, returnall=True)
        mg_df = mg_output["out"]
        mg_df = mg_df[["symbol", "taxid"]]

        # only get the species that we support
        mg_df = mg_df[mg_df["taxid"].isin(inv_tax_ids.keys())]
        mg_df = mg_df.replace({"taxid": inv_tax_ids})

        mg_df["query"] = mg_df.index.values

        # complete missing information
        # TODO --> do it nicer
        for tax_id, organism in inv_tax_ids.items():
            for id in ids:
                if not ((mg_df["query"] == id) & (mg_df["taxid"] == organism)).any():
                    row = pd.DataFrame({"query": [id], "symbol": ["None"], "taxid": [organism]})
                    mg_df = mg_df.append(row, ignore_index=True)

        mapping = pd.DataFrame(
            {"Gene Name": mg_df["query"], "Reduced Gene Name": mg_df["symbol"], "Organism": mg_df["taxid"]})
        mapping = mapping.replace({np.nan: "None"})
        return mapping

    def get_mapping(self, ids, in_type, organism=None, tar_organism=None, ignore_missing=False,
                    reduction_mode="ensembl"):
        """
        Load prefetched mappings to set of IDs and add missing entries.

        :param ids: Set of either protein IDs or gene names
        :param in_type: Type of needed mapping [protein, orthologs, reduced_genes]
        :param organism: Organism the input IDs (should) belong to
        :param tar_organism: (Orthologs mode) Target organism to find the orthologs of
        :param ignore_missing: Bool indicating if not previously fetched IDs should be ignored
        :param reduction_mode: Mode of how to reduce the gene names
        :return: Dataframe with mapping
        """
        # ===== get precalculated =====
        df, missing = self.get_preloaded(in_list=ids, in_type=in_type, organism=organism, tar_organism=tar_organism,
                                         reduction_mode=reduction_mode)
        # ===== get missing =====
        if len(missing) > 0 and not ignore_missing:
            df2 = pd.DataFrame()
            # ==== Filter protein IDs ====
            if in_type == "protein":
                df2 = self.get_uniprot_mapping(ids=missing, organism=organism)
            # ==== Map orthologs ====
            if in_type == "orthologs":
                df2 = self.get_ortholog_mapping(ids=missing, organism=organism, tar_organism=tar_organism)
            # ==== Reduce gene names ====
            if in_type == "reduced_genes":
                df2 = self.get_reduced_mapping(ids=missing, organism=organism, reduction_mode=reduction_mode)
            # ==== Concat if new found ====
            if not df2.empty:
                df = pd.concat([df, df2])
        return df

    # === Check existing mapping entries and return missing ones ====
    def get_preloaded(self, in_list: list, in_type: str, organism=None, tar_organism=None, reduction_mode="ensembl"):
        # ==== Filter protein IDs ====
        if in_type == "protein":
            organisms = {"human": "Homo sapiens (Human)", "rat": "Rattus norvegicus (Rat)",
                         "mouse": "Mus musculus (Mouse)", "rabbit": "Oryctolagus cuniculus (Rabbit)"}
            cur_mapping = self.full_protein_mapping[self.full_protein_mapping["Protein ID"].isin(in_list)]
            if organism is not None:
                cur_mapping = cur_mapping[cur_mapping['Organism'] == organisms[organism]]
            return cur_mapping, list(set(in_list) - set(self.full_protein_mapping["Protein ID"]))
        # ==== Map orthologs ====
        elif in_type == "orthologs":
            cur_mapping = self.full_ortholog_mapping[self.full_ortholog_mapping["source_symbol"].isin(in_list)]
            cur_mapping = cur_mapping[cur_mapping['source_organism'] == organism]
            cur_mapping = cur_mapping[cur_mapping['target_organism'] == tar_organism]
            return cur_mapping, list(set(in_list) - set(cur_mapping["source_symbol"]))
        # ==== Reduce gene names ====
        elif in_type == "reduced_genes":
            cur_mapping = self.full_reduced_gene_mapping[self.full_reduced_gene_mapping["Gene Name"].isin(in_list)]
            cur_mapping = cur_mapping[cur_mapping['Organism'] == organism]
            cur_mapping = cur_mapping[cur_mapping['Mode'] == reduction_mode]
            return cur_mapping, list(set(in_list) - set(cur_mapping["Gene Name"]))
        else:
            return None

    # === Save new mappings to files ====
    # def save_mappings(self, mapping_dir):
    #     mapping_dir = join(here, mapping_dir)
    #     self.full_protein_mapping.to_csv(mapping_dir + "protein_to_genenames.csv", index=False)
    #     self.full_ortholog_mapping.to_csv(mapping_dir + "genenames_to_orthologs.csv", index=False)
    #     self.full_reduced_gene_mapping.to_csv(mapping_dir + "genenames_to_reduced_genenames.csv", index=False)



#!/usr/bin/python3

import csv
import pandas as pd
from .mq_utils import mapping_handler as mh, runner_utils as ru
from .mq_utils.logger import get_remapped_genenames_logging
from .fasta_grepper import grep_header_info
from pathlib import Path
import warnings

warnings.filterwarnings('ignore')


def remap_genenames(data: pd.DataFrame, mode: str, protein_column: str, gene_column: str = None,
                    skip_filled: bool = False, organism: str = None, fasta: str = None, keep_empty: bool = True,
                    res_column: str = None):
    """
    Remap gene names in data file based on chosen mode.

    :param data: Dataframe containing a column with protein IDs and optionally gene names
    :param mode: Mode on how to map gene names
    :param protein_column: Column name with protein IDs
    :param gene_column: Column name with gene names
    :param skip_filled: Set True if rows with already filled gene names should be ignored
    :param organism: Organism to map to
    :param fasta: Fasta file
    :param keep_empty: Set True if empty rows should be kept
    :param res_column: Set column name for remap genenames results. If None, the gene_column will be overridden.
    :return: Remapped data as dataframe
    """
    data_copy = data.copy(deep=True)
    data_copy[protein_column] = data_copy[protein_column].astype("string")

    handler = mh.MappingHandler()
    # ==== Preload info for all IDs ====
    handler.get_mapping(ids=";".join(data_copy[protein_column]).split(";"),
                        in_type="protein", organism=organism)

    # ==== If gene_column does not exist in given dataframe ====
    temp_gene_column = "Remapped Gene Names" if gene_column is None else gene_column
    data_copy[temp_gene_column] = "" if temp_gene_column not in data_copy.columns else data_copy[temp_gene_column].fillna("")

    # ==== Get fasta mapping ====
    if fasta is not None and mode in ['all', 'fasta']:
        fasta_mapping = grep_header_info(fasta=parameters.mapping_file)
        remapped_gene_names = data_copy.apply(
            lambda row: get_fasta_mapping(ids=row[protein_column].split(";"), genename=row[temp_gene_column],
                                          mapping=fasta_mapping, skip_filled=skip_filled), axis=1)
        skip_filled = True
    else:
        remapped_gene_names = data_copy[temp_gene_column]

    # ==== Get uniprot mappings ====
    if mode != 'fasta':
        data_copy["temp"] = remapped_gene_names  # copy into df for apply function
        remapped_gene_names = data_copy.apply(
            lambda row: get_uniprot_mapping(ids=row[protein_column].split(";"), genename=row["temp"],
                                            mode=mode, organism=organism, handler=handler,
                                            skip_filled=skip_filled), axis=1)
        del data_copy["temp"]

    # ==== Logging ====
    log_dict = get_remapped_genenames_logging(original=data_copy[temp_gene_column], remapped=remapped_gene_names,
                                              protein_ids=";".join(data_copy[protein_column]).split(";"),
                                              handler=handler, organism=organism)

    # ==== If target column depending if res_column is set ====
    column = res_column if res_column is not None else temp_gene_column

    # ==== Set remapped gene names to dataFrame ====
    data_copy[column] = remapped_gene_names

    # Remove temporary gene column
    if gene_column is None and res_column is not None:
        data_copy = data_copy.drop(temp_gene_column, axis=1)

    # ==== Remove rows with empty gene names ====
    if keep_empty is False:
        data_copy = data_copy[data_copy[column] != ""]  # remove

    return data_copy, log_dict


def get_fasta_mapping(ids, genename, mapping=None, skip_filled=False):
    """
    Get gene names from fasta file for empty entries or all if skip_filles is set to false.

    :param ids: List of protein ids
    :param genename: Mapped gene name
    :param mapping: Mapping from fasta file
    :param skip_filled: Set True if skip mapping when genename is not empty
    :return: Gene name
    """
    if genename == "" or not skip_filled:
        symbols = set(mapping[mapping["uniprot"].isin(ids)]["symbol"].dropna())
        return ";".join(list(symbols))
    else:
        return genename


def get_uniprot_mapping(ids, genename, mode, handler, organism=None, skip_filled=False):
    """
    Get gene names from uniprot for empty entries or all if skip_filles is set to false.

    :param ids: List of protein ids
    :param genename: Mapped gene name
    :param mode: Mode on how to map gene names
    :param handler: Handler for uniprot mappings
    :param organism: Organism to map to
    :param skip_filled: Set True if skip mapping when genename is not empty
    :return: Gene name
    """
    if genename == "" or not skip_filled:
        if mode == "uniprot_one":
            return get_single_genename(ids=ids, organism=organism, handler=handler)
        elif mode == "uniprot_primary":
            return get_primary_genenames(ids=ids, organism=organism, handler=handler)
        else:
            return get_all_genenames(ids=ids, organism=organism, handler=handler)
    else:
        return genename


def get_single_genename(ids, handler: mh.MappingHandler, organism=None):
    """
    Get most frequent gene name from uniprot.

    :param ids: List of protein IDs
    :param organism: Organism to map to
    :param handler: Handler for uniprot mappings
    :return: Single gene name
    """
    mapping = handler.get_mapping(ids=ids, in_type="protein", organism=organism)
    # ==== Get primary gene name first if only one for all ====
    prim_keys = set(mapping['Gene Names (primary)'].fillna(""))
    if len(prim_keys) == 1:
        return ";".join(prim_keys)
    # ==== Most frequent out of all ====
    mapping['Gene Names'] = mapping['Gene Names'].fillna("")
    gene_names = mapping['Gene Names'].apply(lambda x: set(x.split(";")))
    lst = [x for z in gene_names for x in z if x != ""]
    # ==== Return most frequent ====
    return max(set(lst), key=lst.count)


def get_primary_genenames(ids, handler: mh.MappingHandler, organism=None):
    """
    Get only primary gene names from uniprot.

    :param ids: List of protein IDs
    :param organism: Organism to map to
    :param handler: Handler for uniprot mappings
    :return: Primary gene names
    """
    mapping = handler.get_mapping(ids=ids, in_type="protein", organism=organism)
    if mapping.empty:
        return ""
    else:
        genenames = {x for x in mapping['Gene Names (primary)'].str.split(";").explode() if pd.notna(x)}  # set()
        return ';'.join(genenames)


def get_all_genenames(ids, handler: mh.MappingHandler, organism=None):
    """
    Get all gene names from uniprot.

    :param ids: List of protein IDs
    :param organism: Organism to map to
    :param handler: Handler for uniprot mappings
    :return: All gene names
    """
    mapping = handler.get_mapping(ids=ids, in_type="protein", organism=organism)
    if mapping.empty:
        return ""
    else:
        mapping['Gene Names'] = mapping['Gene Names'].fillna("")
        gene_names_series = mapping['Gene Names'].apply(lambda x: set(x.split(";")))
        genenames = set([x for y in gene_names_series for x in y if x != ""])
        return ';'.join(genenames)


if __name__ == "__main__":
    description = "                  Re-mapp gene names in data file."
    parameters = ru.save_parameters(script_desc=description,
                                    arguments=('d', 'm', 'pc_req', 'gc', 'l', 'or', 'f', 'ke', 'rc', 'o'))
    res, log = remap_genenames(data=parameters.data, mode=parameters.mode, protein_column=parameters.protein_column,
                               gene_column=parameters.gene_column, skip_filled=parameters.fill,
                               organism=parameters.organism, fasta=parameters.fasta_file,
                               keep_empty=parameters.keep_empty,
                               res_column=parameters.res_column)

    res.to_csv(parameters.out_dir + Path(parameters.file_name).stem + "_remapped.txt", header=True,
               index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")

    log["Overview_Log"].to_csv(parameters.out_dir + Path(parameters.file_name).stem + "_remapped_overview_log.txt",
                               header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")
    log["Detailed_Log"].to_csv(
        parameters.out_dir + Path(parameters.file_name).stem + "_remapped_detailed_log.txt",
        header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")



#!/usr/bin/python3

import csv
from pathlib import Path
import pandas as pd
from .mq_utils import mapping_handler as mh, runner_utils as ru
from .mq_utils.logger import get_ortholog_genenames_logging


def map_orthologs(data: pd.DataFrame, gene_column: str, organism: str, tar_organism: str,
                  keep_empty: bool = True, res_column: str = None):
    """
    Map gene names of origin organism to orthologs of target organism.

    :param data: Dataframe containing a column with gene names
    :param gene_column: Column name with gene names
    :param organism: Organism of the input ids
    :param tar_organism: Organism to map to
    :param keep_empty: Set True if empty rows should be kept
    :param res_column: Set column name for ortholog results. If None, the gene_column will be overridden.
    :return: Data as dataframe with ortholog ids
    """
    data_copy = data.copy(deep=True)
    data_copy[gene_column] = data_copy[gene_column].astype("string")

    handler = mh.MappingHandler()
    # ==== Get all existing mappings in one batch ====
    handler.get_mapping(ids=";".join(data_copy[gene_column]).split(";"),
                        in_type="orthologs", organism=organism, tar_organism=tar_organism)

    ortholog_gene_names = data_copy[gene_column].apply(
        lambda x: get_orthologs(ids=x.split(";"), handler=handler, organism=organism, tar_organism=tar_organism))

    # ==== Logging ====
    log_dict = get_ortholog_genenames_logging(original=data_copy[gene_column], orthologs=ortholog_gene_names,
                                              handler=handler, organism=organism, tar_organism=tar_organism)

    # ==== If target column depending if res_column is set ====
    column = res_column if res_column is not None else gene_column

    # ==== Set ortholog gene names to dataframe ====
    data_copy[column] = ortholog_gene_names

    # ==== Remove rows with empty ortholog gene names ====
    if keep_empty is False:
        data_copy = data_copy[data_copy[column] != ""]  # remove

    return data_copy, log_dict


def get_orthologs(ids, handler, organism: str, tar_organism: str):
    """
    Get orthologs of genes from one organism to another.

    :param ids: Set of gene names
    :param handler: Handler for mappings
    :param organism: Organism of the input ids
    :param tar_organism: Organism to map to

    :return:
    """
    mapping = handler.get_mapping(ids=ids, in_type="orthologs", organism=organism,
                                  tar_organism=tar_organism, ignore_missing=True)
    if mapping.empty:
        return ""
    else:
        orthologs = {x for x in mapping['target_symbol'] if pd.notna(x) and x not in ["N/A", "None", None]}
        return ';'.join(orthologs)


if __name__ == "__main__":
    description = "                       Map ortholog gene names in data file."
    parameters = ru.save_parameters(script_desc=description,
                                    arguments=('d','gc_req','or_req','tor_req','ke','rc','o'))
    df, log = map_orthologs(data=parameters.data, gene_column=parameters.gene_column,organism=parameters.organism,
                            tar_organism=parameters.tar_organism, keep_empty=parameters.keep_empty,
                            res_column=parameters.res_column)

    df.to_csv(parameters.out_dir + Path( parameters.file_name ).stem + "_ortholog.txt", header=True, index=False,
              quoting=csv.QUOTE_NONNUMERIC, sep=" ")
    log[ "Overview_Log" ].to_csv(
        parameters.out_dir + Path( parameters.file_name ).stem + "_ortholog_overview_log.txt",
        header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" " )
    log[ "Detailed_Log" ].to_csv(
        parameters.out_dir + Path( parameters.file_name ).stem + "_ortholog_detailed_log.txt",
        header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" " )




#!/usr/bin/python3

import csv
import pandas as pd
from .mq_utils import mapping_handler as mh, runner_utils as ru
from .mq_utils.logger import get_filter_ids_logging
from pathlib import Path


def filter_protein_ids(data: pd.DataFrame, protein_column: str, organism: str = None,
                       rev_con: bool = False, keep_empty: bool = True,
                       reviewed: bool = True, res_column: str = None):
    """
    Filter protein ids in given data by chosen organism.

    :param data: Dataframe containing a column with protein IDs
    :param protein_column: Column name with protein IDs
    :param organism: Organism to map to
    :param rev_con: Bool to indicate if decoy IDs (REV_) and contaminants (CON_) should be kept
    :param keep_empty: Set True if empty rows should be kept
    :param reviewed: Set True if only reviewed protein IDs should be kept
    :param res_column: Set column name for remap genenames results. If None, the gene_column will be overridden.
    :return: Filtered data as dataframe
    """
    data_copy = data.copy(deep=True)
    data_copy[protein_column] = data_copy[protein_column].astype("string")

    handler = mh.MappingHandler()
    # ==== Get all existing mappings in one batch ====
    handler.get_mapping(ids=";".join(data_copy[protein_column]).split(";"),
                        in_type="protein", organism=organism)

    # ==== Filter row wise ====
    filtered_ids = data_copy[protein_column].apply(
        lambda x: get_filtered_ids(ids=x.split(";"), handler=handler, organism=organism, rev_con=rev_con,
                                   reviewed=reviewed))

    # ==== Logging ====
    log_dict = get_filter_ids_logging(original=data_copy[protein_column], filtered=filtered_ids, handler=handler,
                                      organism=organism)

    # ==== If target column depending if res_column is set ====
    column = res_column if res_column is not None else protein_column

    # ==== Set filtered ids to dataframe ====
    data_copy[column] = filtered_ids

    # ==== Save current mapping to files
    # handler.save_mappings(mapping_dir="mappings/")

    # ==== Remove rows with empty protein IDs ====
    if keep_empty is False:
        data_copy = data_copy[data_copy[column] != ""]  # remove

    return data_copy, log_dict


def get_filtered_ids(ids, handler: mh.MappingHandler, organism: str = None, rev_con: bool = False,
                     reviewed: bool = False) -> str:
    """
    Filter given set of protein ids based on organism, decoy/contaminants and/or review status.

    :param ids: Set of protein IDs
    :param handler: MappingHandler object
    :param organism: Organism the IDs should belong to
    :param rev_con: Bool to indicate if decoy and contaminant IDs should be kept
    :param reviewed: Bool to indicate if only reviewed IDs should be kept
    :return: filtered IDs combined into a string
    """
    # ==== Get mapping on protein IDs ====
    mapping = handler.get_mapping(ids=ids, in_type="protein", organism=organism, ignore_missing=True)
    if mapping.empty:
        return ""

    # ==== Keep or remove decoy/contaminants IDs based on flag ====
    keep = set([x for x in ids if x.startswith(("REV", "CON"))]) if rev_con else set()

    # ==== Keep only reviewed IDs based on flag ====
    if reviewed:
        mapping = mapping[mapping['Reviewed'] == "reviewed"]

    # ==== Combine mapped IDs with kept or left decoy IDs ====
    prot_ids = set(mapping['Protein ID']).union(keep)
    return ';'.join(prot_ids)


if __name__ == "__main__":
    description = "        Filter proteins by organism and/or decoy/contaminants names in data file."
    parameters = ru.save_parameters(script_desc=description,
                                    arguments=('d', 'pc_req', 'or', 'rv', 'ke', 'r', 'rc', 'o'))
    df, log = filter_protein_ids(data=parameters.data, protein_column=parameters.protein_column,
                                 organism=parameters.organism, rev_con=parameters.rev_con,
                                 keep_empty=parameters.keep_empty, reviewed=parameters.reviewed,
                                 res_column=parameters.res_column)
    df.to_csv(parameters.out_dir + Path(parameters.file_name).stem + "_filtered.txt", header=True, index=False,
              quoting=csv.QUOTE_NONNUMERIC, sep=" ")

    log["Overview_Log"].to_csv(
        parameters.out_dir + Path(parameters.file_name).stem + "_filtered_overview_log.txt",
        header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")
    log["Detailed_Log"].to_csv(
        parameters.out_dir + Path(parameters.file_name).stem + "_filtered_detailed_log.txt",
        header=True, index=False, quoting=csv.QUOTE_NONNUMERIC, sep=" ")



import setuptools
from setuptools import find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="proharmed",
    version="0.0.3",
    author="Klaudia Adamowicz",
    author_email='klaudia.adamowicz@uni-hamburg.de',
    url='http://pypi.python.org/pypi/proharmed/',
    license='LICENSE',
    description="Collection of scripts for ProHarMeD",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Intended Audience :: Science/Research',
        # "License :: OSI Approved :: MIT License",
        # "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.7',
    install_requires=[
        "pandas>=1.2.0",
        "requests>=2.26.0",
        "gprofiler-official>=1.0.0",
        "httplib2>=0.21.0",
        "ratelimit>=2.2.1",
        "mygene",
        "seaborn",
        "psutil",
        "upsetplot"
    ]
)
